"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import importlib.metadata

__title__: str = "polar-sdk"
__version__: str = "0.17.3"
__openapi_doc_version__: str = "0.1.0"
__gen_version__: str = "2.578.0"
__user_agent__: str = "speakeasy-sdk/python 0.17.3 2.578.0 0.1.0 polar-sdk"

try:
    if __package__ is not None:
        __version__ = importlib.metadata.version(__package__)
except importlib.metadata.PackageNotFoundError:
    pass
