"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .eventname import EventName, EventNameTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceEventNameTypedDict(TypedDict):
    items: List[EventNameTypedDict]
    pagination: PaginationTypedDict


class ListResourceEventName(BaseModel):
    items: List[EventName]

    pagination: Pagination
