"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class PaymentMethodGenericTypedDict(TypedDict):
    id: str
    type: str
    created_at: datetime
    default: bool


class PaymentMethodGeneric(BaseModel):
    id: str

    type: str

    created_at: datetime

    default: bool
