"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpricecustom import ProductPriceCustom, ProductPriceCustomTypedDict
from .productpricefixed import ProductPriceFixed, ProductPriceFixedTypedDict
from .productpricefree import ProductPriceFree, ProductPriceFreeTypedDict
from .productpricemeteredunit import (
    ProductPriceMeteredUnit,
    ProductPriceMeteredUnitTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ProductPriceTypedDict = TypeAliasType(
    "ProductPriceTypedDict",
    Union[
        ProductPriceFreeTypedDict,
        ProductPriceFixedTypedDict,
        ProductPriceCustomTypedDict,
        ProductPriceMeteredUnitTypedDict,
    ],
)


ProductPrice = TypeAliasType(
    "ProductPrice",
    Union[
        ProductPriceFree, ProductPriceFixed, ProductPriceCustom, ProductPriceMeteredUnit
    ],
)
