"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionprorationbehavior import SubscriptionProrationBehavior
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrganizationSubscriptionSettingsTypedDict(TypedDict):
    allow_multiple_subscriptions: bool
    allow_customer_updates: bool
    proration_behavior: SubscriptionProrationBehavior


class OrganizationSubscriptionSettings(BaseModel):
    allow_multiple_subscriptions: bool

    allow_customer_updates: bool

    proration_behavior: SubscriptionProrationBehavior
