"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerbenefitgrantgithubrepositorypropertiesupdate import (
    CustomerBenefitGrantGitHubRepositoryPropertiesUpdate,
    CustomerBenefitGrantGitHubRepositoryPropertiesUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CustomerBenefitGrantGitHubRepositoryUpdateTypedDict(TypedDict):
    properties: CustomerBenefitGrantGitHubRepositoryPropertiesUpdateTypedDict
    benefit_type: Literal["github_repository"]


class CustomerBenefitGrantGitHubRepositoryUpdate(BaseModel):
    properties: CustomerBenefitGrantGitHubRepositoryPropertiesUpdate

    BENEFIT_TYPE: Annotated[
        Annotated[
            Literal["github_repository"],
            AfterValidator(validate_const("github_repository")),
        ],
        pydantic.Field(alias="benefit_type"),
    ] = "github_repository"
