"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .eventsource import EventSource
from datetime import datetime
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class EventNameTypedDict(TypedDict):
    name: str
    r"""The name of the event."""
    source: EventSource
    occurrences: int
    r"""Number of times the event has occurred."""
    first_seen: datetime
    r"""The first time the event occurred."""
    last_seen: datetime
    r"""The last time the event occurred."""


class EventName(BaseModel):
    name: str
    r"""The name of the event."""

    source: EventSource

    occurrences: int
    r"""Number of times the event has occurred."""

    first_seen: datetime
    r"""The first time the event occurred."""

    last_seen: datetime
    r"""The last time the event occurred."""
