"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class CountAggregationTypedDict(TypedDict):
    func: Literal["count"]


class CountAggregation(BaseModel):
    FUNC: Annotated[
        Annotated[Optional[Literal["count"]], AfterValidator(validate_const("count"))],
        pydantic.Field(alias="func"),
    ] = "count"
