"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class BenefitGrantDownloadablesPropertiesTypedDict(TypedDict):
    files: NotRequired[List[str]]


class BenefitGrantDownloadablesProperties(BaseModel):
    files: Optional[List[str]] = None
