"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class BenefitDownloadablesSubscriberPropertiesTypedDict(TypedDict):
    active_files: List[str]


class BenefitDownloadablesSubscriberProperties(BaseModel):
    active_files: List[str]
