"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomFieldSelectOptionTypedDict(TypedDict):
    value: str
    label: str


class CustomFieldSelectOption(BaseModel):
    value: str

    label: str
