"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CustomerCustomerMeterSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    MODIFIED_AT = "modified_at"
    MINUS_MODIFIED_AT = "-modified_at"
    METER_ID = "meter_id"
    MINUS_METER_ID = "-meter_id"
    METER_NAME = "meter_name"
    MINUS_METER_NAME = "-meter_name"
    CONSUMED_UNITS = "consumed_units"
    MINUS_CONSUMED_UNITS = "-consumed_units"
    CREDITED_UNITS = "credited_units"
    MINUS_CREDITED_UNITS = "-credited_units"
    BALANCE = "balance"
    MINUS_BALANCE = "-balance"
