"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class BenefitGrantMeterCreditPropertiesTypedDict(TypedDict):
    last_credited_meter_id: str
    last_credited_units: int
    last_credited_at: str


class BenefitGrantMeterCreditProperties(BaseModel):
    last_credited_meter_id: str

    last_credited_units: int

    last_credited_at: str
