"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerSubscriptionUpdateProductTypedDict(TypedDict):
    product_id: str
    r"""Update subscription to another product."""


class CustomerSubscriptionUpdateProduct(BaseModel):
    product_id: str
    r"""Update subscription to another product."""
