"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class BenefitMeterCreditSubscriberPropertiesTypedDict(TypedDict):
    r"""Properties available to subscribers for a benefit of type `meter_unit`."""

    units: int
    meter_id: str


class BenefitMeterCreditSubscriberProperties(BaseModel):
    r"""Properties available to subscribers for a benefit of type `meter_unit`."""

    units: int

    meter_id: str
