"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EventNamesSortProperty(str, Enum):
    NAME = "name"
    MINUS_NAME = "-name"
    OCCURRENCES = "occurrences"
    MINUS_OCCURRENCES = "-occurrences"
    FIRST_SEEN = "first_seen"
    MINUS_FIRST_SEEN = "-first_seen"
    LAST_SEEN = "last_seen"
    MINUS_LAST_SEEN = "-last_seen"
