"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptioncancel import SubscriptionCancel, SubscriptionCancelTypedDict
from .subscriptionupdateproduct import (
    SubscriptionUpdateProduct,
    SubscriptionUpdateProductTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


SubscriptionUpdateTypedDict = TypeAliasType(
    "SubscriptionUpdateTypedDict",
    Union[SubscriptionUpdateProductTypedDict, SubscriptionCancelTypedDict],
)


SubscriptionUpdate = TypeAliasType(
    "SubscriptionUpdate", Union[SubscriptionUpdateProduct, SubscriptionCancel]
)
