"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class ProductPriceMeterTypedDict(TypedDict):
    r"""A meter associated to a metered price."""

    id: str
    r"""The ID of the object."""
    name: str
    r"""The name of the meter."""


class ProductPriceMeter(BaseModel):
    r"""A meter associated to a metered price."""

    id: str
    r"""The ID of the object."""

    name: str
    r"""The name of the meter."""
