"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customfieldselectoption import (
    CustomFieldSelectOption,
    CustomFieldSelectOptionTypedDict,
)
from polar_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CustomFieldSelectPropertiesTypedDict(TypedDict):
    options: List[CustomFieldSelectOptionTypedDict]
    form_label: NotRequired[str]
    form_help_text: NotRequired[str]
    form_placeholder: NotRequired[str]


class CustomFieldSelectProperties(BaseModel):
    options: List[CustomFieldSelectOption]

    form_label: Optional[str] = None

    form_help_text: Optional[str] = None

    form_placeholder: Optional[str] = None
