"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class EventsIngestResponseTypedDict(TypedDict):
    inserted: int
    r"""Number of events inserted."""


class EventsIngestResponse(BaseModel):
    inserted: int
    r"""Number of events inserted."""
