"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BenefitGrantLicenseKeysPropertiesTypedDict(TypedDict):
    license_key_id: NotRequired[str]
    display_key: NotRequired[str]


class BenefitGrantLicenseKeysProperties(BaseModel):
    license_key_id: Optional[str] = None

    display_key: Optional[str] = None
