"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerupdate import CustomerUpdate, CustomerUpdateTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CustomersUpdateExternalRequestTypedDict(TypedDict):
    external_id: str
    r"""The customer external ID."""
    customer_update: CustomerUpdateTypedDict


class CustomersUpdateExternalRequest(BaseModel):
    external_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The customer external ID."""

    customer_update: Annotated[
        CustomerUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
