"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customermeter import CustomerMeter, CustomerMeterTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceCustomerMeterTypedDict(TypedDict):
    items: List[CustomerMeterTypedDict]
    pagination: PaginationTypedDict


class ListResourceCustomerMeter(BaseModel):
    items: List[CustomerMeter]

    pagination: Pagination
