"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitcustom import BenefitCustom, BenefitCustomTypedDict
from .benefitdiscord import BenefitDiscord, BenefitDiscordTypedDict
from .benefitdownloadables import BenefitDownloadables, BenefitDownloadablesTypedDict
from .benefitgithubrepository import (
    BenefitGitHubRepository,
    BenefitGitHubRepositoryTypedDict,
)
from .benefitlicensekeys import BenefitLicenseKeys, BenefitLicenseKeysTypedDict
from .benefitmetercredit import BenefitMeterCredit, BenefitMeterCreditTypedDict
from typing import Union
from typing_extensions import TypeAliasType


BenefitTypedDict = TypeAliasType(
    "BenefitTypedDict",
    Union[
        BenefitCustomTypedDict,
        BenefitDiscordTypedDict,
        BenefitGitHubRepositoryTypedDict,
        BenefitDownloadablesTypedDict,
        BenefitLicenseKeysTypedDict,
        BenefitMeterCreditTypedDict,
    ],
)


Benefit = TypeAliasType(
    "Benefit",
    Union[
        BenefitCustom,
        BenefitDiscord,
        BenefitGitHubRepository,
        BenefitDownloadables,
        BenefitLicenseKeys,
        BenefitMeterCredit,
    ],
)
