"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customersubscription import CustomerSubscription, CustomerSubscriptionTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceCustomerSubscriptionTypedDict(TypedDict):
    items: List[CustomerSubscriptionTypedDict]
    pagination: PaginationTypedDict


class ListResourceCustomerSubscription(BaseModel):
    items: List[CustomerSubscription]

    pagination: Pagination
